#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import configparser

class ConfigReader:
    """
    @brief ConfigReader class for reading configuration parameters from a file.

    This class uses the configparser module to read configuration parameters
    from a specified configuration file.

    Usage:
    ```
    config_reader = ConfigReader('path/to/config.ini')
    image_name = config_reader.getImageName()
    projected_image_width = config_reader.getProjectedImageWidth()
    projected_overlap_width = config_reader.getProjectedOverlapWidth()
    gamma = config_reader.getGamma()
    image_side = config_reader.getImageSide()
    ```

    The configuration file should have a [DEFAULT] section with the following keys:
    - 'image_name': Name of the image.
    - 'projected_image_width': Width of the projected image.
    - 'projected_overlap_width': Width of the projected image overlap.
    - 'gamma': Gamma value for image processing.
    - 'image_side': Side of the image.

    @note Ensure that the specified configuration file exists and has the required keys.
    """

    def __init__(self, config_path):
        """
        @brief Constructor for ConfigReader class.

        @param config_path: Path to the configuration file.
        """
        self.config_parser = configparser.ConfigParser()
        self.config_parser.read(config_path)

    def getImageName(self):
        """
        @brief Get the image name from the configuration.

        @return: Image name as a string.
        """
        return str(self.config_parser['DEFAULT']['image_name'])

    def getProjectedImageWidth(self):
        """
        @brief Get the projected image width from the configuration.

        @return: Projected image width as an integer.
        """
        return int(self.config_parser['DEFAULT']['projected_image_width'])

    def getProjectedOverlapWidth(self):
        """
        @brief Get the projected overlap width from the configuration.

        @return: Projected overlap width as an integer.
        """
        return int(self.config_parser['DEFAULT']['projected_overlap_width'])

    def getGamma(self):
        """
        @brief Get the gamma value from the configuration.

        @return: Gamma value as a float.
        """
        return float(self.config_parser['DEFAULT']['gamma'])

    def getImageSide(self):
        """
        @brief Get the image side from the configuration.

        @return: Image side as an integer.
        """
        return int(self.config_parser['DEFAULT']['image_side'])