#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@file
@brief This script defines the MainDisplay class, which handles reading, setting, and displaying images using the OpenCV library.

@date January 18, 2024
"""

import cv2

class MainDisplay:
    """
    @brief The MainDisplay class provides functionalities for image handling and display.

    @details
    The class includes methods for:
    - Reading an image from a specified path.
    - Setting the result image.
    - Displaying the result image using the OpenCV library.

    @note
    Ensure that the OpenCV library is installed (`pip install opencv-python`) before using this class.

    @date January 18, 2024
    """

    def __init__(self):
        """
        @brief Initializes a MainDisplay object.

        @details
        The result_image attribute is initialized to None.
        """
        self.result_image = None

    def readImage(self, image_path):
        """
        @brief Reads an image from the specified path.

        @param image_path: The path to the image file.

        @return The read image.
        """
        image = cv2.imread(image_path)
        return image

    def setImage(self, image):
        """
        @brief Sets the result image.

        @param image: The image to set as the result image.

        @return The set result image.
        """
        self.result_image = image
        return self.result_image

    def saveImage(self):
        """
        @brief Displays and saves the result image.

        @details
        The result image is displayed in a window with the window name indicating the displayed image.
        The window remains open until a key is pressed. Then, it is closed, and the program continues.

        @note
        Ensure that the OpenCV window is closed before proceeding with the execution of other code.

        @warning
        This method uses blocking code (cv2.waitKey(0)) that might lead to issues in certain scenarios.
        """
        # display image. (window name, image to display)
        cv2.imshow(f"Displaying {self.result_image}", self.result_image)
        cv2.waitKey(0)
        cv2.destroyAllWindows()