import cv2
import numpy as np

class MaskCreator:
    """
    @brief The MaskCreator class performs image modification, including creating masks, gamma correction, alpha blending, and intensity modification.

    @details
    The class includes methods for:
    - Creating masks based on specified parameters.
    - Applying gamma correction to the image.
    - Performing alpha blending on the image based on the image side.
    - Modifying image intensity based on the image side.

    @note
    Ensure that the OpenCV library is installed (`pip install opencv-python`) before using this class.

    @date January 18, 2024
    """

    def __init__(self, image):
        """
        @brief Initializes a MaskCreator object.

        @param image: The input image on which modifications will be applied.
        """
        self.__image = image
        self.__alpha_gradient = None
        self.__gamma_corrected = None
        self.result_image = None
        self.__mask = None
        
    def create_mask(self, image_side, mask_width, image_width):
        """
        @brief Creates a mask based on specified parameters.

        @param image_side: The side of the image (0 for left, 1 for right).
        @param mask_width: The width of the mask.
        @param image_width: The width of the input image.
        """
        self.__mask = self.__image.shape[1] * mask_width // image_width
        if image_side == 1:
            self.__alpha_gradient = np.linspace(1, 0, self.__mask)
        elif image_side == 0:
            self.__alpha_gradient = np.linspace(0, 1, self.__mask)

    def gammaCorrection(self, gamma):
        """
        @brief Applies gamma correction to the image.

        @param gamma: The gamma value for correction.
        """
        self.__gamma_corrected = np.uint8(cv2.pow(self.__image / 255.0, gamma) * 255)
        
    def alpha_blending(self, image_side):
        """
        @brief Performs alpha blending on the image based on the image side.

        @param image_side: The side of the image (0 for left, 1 for right).
        """
        if image_side == 1:
            for col in range(self.__mask):
                alpha = self.__alpha_gradient[-self.__mask + col]
                self.result_image[:, col] = alpha * self.__gamma_corrected[:, col] + (1 - alpha) * self.result_image[:, col]
        elif image_side == 0:
            for col in range(self.__mask):
                alpha = self.__alpha_gradient[-self.__mask + col]
                self.result_image[:, -self.__mask + col] = alpha * self.__gamma_corrected[:, -self.__mask + col] + (1 - alpha) * self.result_image[:, -self.__mask + col]
        
    def mod_intensity(self, image_side):
        """
        @brief Modifies image intensity based on the image side.

        @param image_side: The side of the image (0 for left, 1 for right).
        """
        if image_side == 1:
            for col in range(self.__mask):
                intensity_factor = 1.0 - (self.__mask - col) / self.__mask
                self.result_image[:, col] = (self.result_image[:, col] * intensity_factor).astype(np.uint8)
        elif image_side == 0:
            for col in range(self.__mask):
                intensity_factor = 1.0 - col / self.__mask
                self.result_image[:, -self.__mask + col] = (self.result_image[:, -self.__mask + col] * intensity_factor).astype(np.uint8)
